''*****************************
''*  Graphics Demo            *
''*  (C) 2005 Parallax, Inc.  *
''*****************************

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  _free = ($3000 + $3000) >> 2          'accomodate bitmap buffers
  _stack = $100                         'insure sufficient stack

  x_tiles = 16
  y_tiles = 12

  paramcount = 14       
  bitmap_base = $2000
  display_base = $5000
 
VAR

  long  mousex, mousey

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

OBJ

  tv    : "tv"
  gr    : "graphics"


PUB start | x, y, i, c, k

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from $00 to $0F
    case i
      5..10 : c := $01000000 * (i - 5) + $02020507
      other  : c := $0E000704
    'c := $02075D05
    colors[i] := c

  repeat i from $10 to $1F
    colors[i] := $10100000 * (i & $F) + $0B0A0507
  repeat i from $20 to $2F
    colors[i] := $10100000 * (i & $F) + $0D0C0507
  repeat i from $30 to $3F
    colors[i] := $10100000 * (i & $F) + $0E0E0507

  'init tile screen
  repeat x from 0 to tv_hc - 1
    repeat y from 0 to tv_vc - 1
      case y
        0..2 : i := $30 + x
        3..4 : i := $20 + x
        5..6 : i := $10 + x
        8    : i := x 
        other:  i := 0
      screen[x + y * tv_hc] := i << 10 + display_base >> 6 + x * tv_vc + y

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 0, 0, bitmap_base)

 

  repeat

    'clear bitmap
    gr.clear

    'draw color samples

    'draw dither samples  <---------
    gr.color(3)
    gr.width(0)

    repeat x from 0 to 15 
      repeat y from 1 to 14 step 3
        gr.plot(y+(x*16), 183-6)
        gr.line(y+(x*16),183+7)

    
    repeat x from 0 to 15 
      repeat y from 1 to 14 step 2
        gr.plot(y+(x*16), 183-6-16)
        gr.line(y+(x*16),183+7-16)
        
    gr.width(29)
    'draw gradient samples
    repeat y from 2 to 6
      gr.color(y & 1 | 2)
      repeat x from 0 to 15
        gr.plot(x << 4 + 7, 183 - y << 4)
    'draw monochrome samples
    gr.color(3)
    repeat x from 5 to 10
      gr.plot(x << 4 + 7, 55) 


    
    repeat x from 0 to 20
      gr.colorwidth(1,0)
      gr.plot(x*2, 0)
      gr.line(x*2,60)

    repeat x from 0 to 20
      gr.colorwidth(3,0)
      gr.plot(x*2+1, 0)
      gr.line(x*2+1,60)

    gr.colorwidth(3,3)
    gr.plot(0, 30)
    gr.line(40,30)
    gr.colorwidth(1,3)
    gr.plot(0, 40)
    gr.line(40,40)


    repeat x from 20 to 40 step 4
      gr.colorwidth(1,4)
      gr.plot(x*2, 0)
      gr.line(x*2,60)

      
    'show appropriate message
    if c => 0
      gr.colorwidth(3, 10)
      gr.textmode(3, 3, 8, %0101)
      hexstring[0] := hex(c >> 4)
      hexstring[1] := hex(c & $F)
      gr.text(128, 17, @colorstring)
      colors.byte[2] := c
      gr.colorwidth(2, 6)
      gr.text(128, 17, @colorstring)       
     
    'copy bitmap to display
    gr.copy(display_base)


PRI hex(value) : chr

  return lookupz(value : "0".."9", "A".."F")

DAT

tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    4               'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    60_000_000      'broadcast
                        long    0               'auralcog


pixdef                  word                    'arrow pointer
                        byte    1,5,0,4
                        word    %%11110000
                        word    %%11100000
                        word    %%11110000
                        word    %%10111000
                        word    %%00011000

colorstring             byte    "COLOR "                        
hexstring               byte    "00",0